/*  -------------------------------------------------------------------------
    Loccalc V2.1, written by Marco Bersani, IK2PIH, Stradella,
    Italy, on December 2003; uses ncurses and requires a 80x24 color terminal.
    Loccalc is a clone of Locator Calculator V2.0, fully re-written
    in C for the Linux OS.

    Locator Calculator V2.0 was a free software written in GW-Basic
    by Lars Berg, SM0CCM,  Jarfalla, Sweden on 15 September 1987, and
    released by the author as a public domain software with no restrictions.

    Loccalc is a program that calculates longitude and latitude from a 4 or 6
    characters world wide locator, calculates distance (kilometers and miles)
    between two locators or calculates a locator from longitude and latitude.
    You will be prompted for all inputs and all inputs are validity checked.

    Local earth radius is calculated from latitude. Then, for beam heading and
    distance calculation, the surface of the earth is supposed to be spherical.

    --------------------------------------------------------------------------
    Copyright 2002-2003, Marco Bersani, bermar2@inwind.it or mbers@libero.it

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program (see the file COPYING which accompany
    this source file);  if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ----------------------------------------------------------------------- */





#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<ncurses.h>

#ifndef PI
# define PI 3.141592654
#endif

#define IS_LONGITUDE 0
#define IS_LATITUDE  1







int check_valid_locator(char *locator);
void calculate_longitude_and_latitude_from_locator(char *locator, double *lon, double *lat);
void calculate_beam_heading_and_distance(double lon1, double lat1, double lon2, double lat2, double *cb, double *di, double *dmi);
void convert_character_to_upper_case(char *string);
void calculate_locator_from_longitude_and_latitude(char *locator, double lon, double lat);
int get_decimal_degrees_from_input_string(char *inputstring, double *result, char t);
double local_earth_radius(double latitude);




int main()
{
/* -------------- MAIN ROUTINE --------------------- */

char my_locator[6];
char other_locator[6];
char inputstring[7];
double my_lon,my_lat,other_lon,other_lat,cb,di,dmi;
char choice;
int retval,righe,colonne,k;
WINDOW *myloc_win, *otherloc_win, *calc_win, *lon_win, *lat_win, *res_win;




initscr();                                 /* Start ncurses */

if(has_colors() == FALSE)                  /* Exit if terminal does not support color */
     {endwin();
      printf("You terminal does not support color.\n");
      exit(1);}

start_color();                             /* Start color */
init_pair(1, COLOR_WHITE, COLOR_BLUE);     /* Define color pairs */
init_pair(2, COLOR_BLACK, COLOR_CYAN);
init_pair(3, COLOR_GREEN, COLOR_BLUE);
init_pair(4, COLOR_BLACK, COLOR_YELLOW);
init_pair(5, COLOR_CYAN, COLOR_BLUE);
init_pair(6, COLOR_YELLOW, COLOR_BLUE);
bkgdset(COLOR_PAIR(1));                    /* Set default color pair to COLOR_PAIR(1) */

getmaxyx(stdscr,righe,colonne);            /* get number of rows and cols */

if(righe<24 || colonne<80)
     {endwin();
      printf("This software requires minimum 80x24 text-mode terminal.\nPlease enlarge your terminal!\n");
      exit(1);}

k=(colonne-80)/2;                          /* left margin */


cbreak();                                  /* Allow break with CTRL-C */
noecho();                                  /* Set echo = OFF */





/* PRINT THE MAIN MENU AND WAIT FOR A VALID CHOICE */
do
 {clear();
  attron(COLOR_PAIR(2));
  mvprintw(0,k+6," ***  IK2PIH LOCCALC V2.1 - a clone of SM0CCM LOCATOR CALCULATOR *** ");
  mvprintw(1,k+6,"                              MAIN MENU                              ");
  attroff(COLOR_PAIR(2));
  attron(COLOR_PAIR(3));
  mvprintw(4,k+5,"Loccalc, version 2.1. - Copyright (C) 2002, 2003 Marco Bersani");
  mvprintw(5,k+5,"This is free software; see the source code for copying conditions.");
  mvprintw(6,k+5,"There is ABSOLUTELY NO WARRANTY; not even for MERCHANTIBILITY or");
  mvprintw(7,k+5,"FITNESS FOR A PARTICULAR PURPOSE.  For details, see \"WARRANTY\".");
  mvprintw(8,k+5,"Report bugs to <bermar2@inwind.it>.");
  attroff(COLOR_PAIR(3));
  attron(COLOR_PAIR(4));
  mvprintw(11,k+24,"----------MAIN MENU-----------");
  attroff(COLOR_PAIR(4));
  mvprintw(12,k+24,"1) BEAM HEADING AND DISTANCE");
  mvprintw(13,k+24,"2) CONVERT LONG/LAT TO LOCATOR");
  mvprintw(14,k+24,"3) WARRANTY");
  mvprintw(15,k+24,"4) ABOUT");
  mvprintw(16,k+24,"5) END");
  mvprintw(18,k+24,"MAKE YOUR CHOICE (1-5)> ");
  refresh();

  choice=0;

 while((choice<'1') || (choice>'5'))
      {choice=getch();}

 echo();                              /* Set echo = ON */








 /* CALCULATE BEAM HEADING AND DISTANCE SECTION*/

  if(choice=='1')
   {
     clear();
     attron(COLOR_PAIR(2));
     mvprintw(0,k+6," ***  IK2PIH LOCCALC V2.1 - a clone of SM0CCM LOCATOR CALCULATOR *** ");
     mvprintw(1,k+6,"                    BEAM HEADING AND DISTANCE                        ");
     attroff(COLOR_PAIR(2));
     refresh();

     /* Create my-locator window */
     myloc_win = newwin(5, 48, 5, 16+k);
     wbkgdset(myloc_win,COLOR_PAIR(1));
     wclear(myloc_win);


     /* Get my locator from my-locator window */
     do {
        wattron(myloc_win,COLOR_PAIR(4));
        wprintw(myloc_win,"----------- INSERT YOUR LOCATOR --------------\n");
        wattroff(myloc_win,COLOR_PAIR(4));
        wprintw(myloc_win,"(TO QUIT: ENTER \"END\" INSTEAD OF YOUR LOCATOR)\n");
        wprintw(myloc_win,"YOUR LOCATOR? ");
        wrefresh(myloc_win);
        wgetstr(myloc_win,my_locator);                  /* Read locator from keyboard */
        convert_character_to_upper_case(my_locator);    /* Convert locator in upper case */
        retval=check_valid_locator(my_locator);         /* Check locator validity */


        if(retval==0)
            {wprintw(myloc_win,"\nWRONG LOCATOR!      PRESS ANY KEY TO CONTINUE");
             wrefresh(myloc_win);
             wgetch(myloc_win);
             wclear(myloc_win);}
        }
     while(retval==0);



     /* Calculate and print my lon and lat in my-locator window */
     if(retval==-1)
      {calculate_longitude_and_latitude_from_locator(my_locator, &my_lon, &my_lat);
       wclear(myloc_win);
       wattron(myloc_win,COLOR_PAIR(4));
       mvwprintw(myloc_win,0,7,"----------YOUR QTH-----------");
       wattroff(myloc_win,COLOR_PAIR(4));
       mvwprintw(myloc_win,1,7,"YOUR LOCATOR IS   = %s",my_locator);
       mvwprintw(myloc_win,2,7,"YOUR LONGITUDE IS = %.2f DEG",my_lon);
       mvwprintw(myloc_win,3,7,"YOUR LATITUDE IS  = %.2f DEG",my_lat);
       mvwprintw(myloc_win,4,7,"YOUR LOCAL EARTH RADIUS = %.1f km",local_earth_radius(my_lat));
       wrefresh(myloc_win);



      /* Create other-locator window */
      otherloc_win = newwin(5, 48, 11, 1+k);
      wbkgdset(otherloc_win,COLOR_PAIR(1));
      wclear(otherloc_win);


      /* Create calculations window */
      calc_win = newwin(11, 27, 11, 51+k);
      wbkgdset(calc_win,COLOR_PAIR(1));
      wclear(calc_win);


      /* Get other locator from other-locator window */
       do
       {
        do {
           wclear(otherloc_win);
           wattron(otherloc_win,COLOR_PAIR(4));
           wprintw(otherloc_win,"--------------INSERT A LOCATOR-------------\n");
           wattroff(otherloc_win,COLOR_PAIR(4));
           wprintw(otherloc_win,"(TO QUIT: ENTER \"END\" INSTEAD OF A LOCATOR)\nLOCATOR? ");
           wrefresh(otherloc_win);
           wgetstr(otherloc_win,other_locator);              /* Read locator from keyboard */
           convert_character_to_upper_case(other_locator);   /* Convert locator in upper case */
           retval=check_valid_locator(other_locator);        /* Check locator validity */

           if(retval==0)
              {wprintw(otherloc_win,"\nWRONG LOCATOR!   PRESS ANY KEY TO CONTINUE");
               wrefresh(otherloc_win);
               wgetch(otherloc_win);
               wclear(otherloc_win);}

           }
       while(retval==0);     /* Repeat if input is not valid */



       /* Calculate results and print them in calculations window */
       if(retval==-1)
           {calculate_longitude_and_latitude_from_locator(other_locator, &other_lon, &other_lat);
            calculate_beam_heading_and_distance(my_lon, my_lat, other_lon, other_lat, &cb, &di, &dmi);
            wclear(calc_win);
            wattron(calc_win,COLOR_PAIR(4));
            wprintw(calc_win,"CALCULATIONS FOR %s\n\n",other_locator);
            wattroff(calc_win,COLOR_PAIR(4));
            wprintw(calc_win,"LONGITUDE = %.2f DEG\n\n",other_lon);
            wprintw(calc_win,"LATITUDE  = %.2f DEG\n\n",other_lat);
            wprintw(calc_win,"BEAM HEADING = %.1f DEG\n\nDISTANCE  = %.1f km\n\nDISTANCE  = %.1f MLS",cb,di,dmi);
            wrefresh(calc_win);}

       }
       while(retval!=-2);    /* Repeat if locator != END */



       delwin(calc_win);     /* Destroy my-locator, other-locator and calculations window */
       delwin(otherloc_win);
       delwin(myloc_win);
     }
   }
/* END OF CALCULATE BEAM HEADING AND DISTANCE SECTION*/








/* CONVERT LONG/LAT TO LOCATOR SECTION*/

  if(choice=='2')
   {
     clear();
     attron(COLOR_PAIR(2));
     mvprintw(0,k+6," ***  IK2PIH LOCCALC V2.1 - a clone of SM0CCM LOCATOR CALCULATOR *** ");
     mvprintw(1,k+6,"                CONVERT LONGITUDE/LATITUDE TO LOCATOR                ");
     attroff(COLOR_PAIR(2));
     mvprintw(4,k+15,"INPUT FORMAT FOR LONGITUDE / LATITUDE = ");
     mvprintw(5,k+15,"Where -     = W/E/N/S  and      = degrees.");
     mvprintw(6,k+15,"            = .dd for decimal degrees.");
     mvprintw(7,k+15,"            = 'dd for minutes.");
     attron(COLOR_PAIR(5));
     mvprintw(4,k+56,"C");
     mvprintw(5,k+25,"C");
     attron(COLOR_PAIR(3));
     mvprintw(4,k+57,"nnn");
     mvprintw(5,k+43,"nnn");
     attron(COLOR_PAIR(6));
     mvprintw(4,k+60,"sdd");
     mvprintw(6,k+23,"sdd");
     mvprintw(7,k+23,"sdd");
     attroff(COLOR_PAIR(6));
     refresh();

     /* Create longitude window */
     lon_win=newwin(5,38,9,k+1);
     wbkgdset(lon_win,COLOR_PAIR(1));


     /* Get longitude */
     do
       {wclear(lon_win);
        wattron(lon_win,COLOR_PAIR(4));
        wprintw(lon_win,"ENTER LONGITUDE   (Cnnnsdd)    \n");
        wattroff(lon_win,COLOR_PAIR(4));
        wprintw(lon_win,"C = W for West, E for East.\n> ");
        wrefresh(lon_win);
        wgetstr(lon_win,inputstring);
        convert_character_to_upper_case(inputstring);
        retval=get_decimal_degrees_from_input_string(inputstring, &my_lon, IS_LONGITUDE);
        if(retval==0)
            {wprintw(lon_win,"WRONG LONGITUDE!\nPRESS ANY KEY TO CONTINUE");
             wrefresh(lon_win);
             wgetch(lon_win);}
       }
     while(retval==0);


     /* Create latitude window */
     lat_win=newwin(5,38,16,k+1);
     wbkgdset(lat_win,COLOR_PAIR(1));


     /* Get latitude */
     do
       {wclear(lat_win);
        wattron(lat_win,COLOR_PAIR(4));
        wprintw(lat_win,"ENTER LATITUDE    (Cnnnsdd)    \n");
        wattroff(lat_win,COLOR_PAIR(4));
        wprintw(lat_win,"C = N for North, S for South.\n> ");
        wrefresh(lat_win);
        wgetstr(lat_win,inputstring);
        convert_character_to_upper_case(inputstring);
        retval=get_decimal_degrees_from_input_string(inputstring, &my_lat, IS_LATITUDE);

        if(retval==0)
            {wprintw(lat_win,"WRONG LATITUDE!\nPRESS ANY KEY TO CONTINUE");
             wrefresh(lat_win);
             wgetch(lat_win);}
       }
      while(retval==0);


     /* Calculate locator from lon and lat and print it in the results window */
     calculate_locator_from_longitude_and_latitude(my_locator, my_lon, my_lat);
     res_win=newwin(10,35,9,k+43);      /* Create results window */
     wbkgdset(res_win,COLOR_PAIR(1));
     wclear(res_win);
     wattron(res_win,COLOR_PAIR(4));
     wprintw(res_win,"----------YOUR QTH-----------\n");
     wattroff(res_win,COLOR_PAIR(4));
     wprintw(res_win,"\nYOUR LOCATOR IS   = %s\nYOUR LONGITUDE IS = %.2f DEG\nYOUR LATITUDE IS  = %.2f DEG\n\n",my_locator,my_lon,my_lat);
     wprintw(res_win,"\nPRESS ANY KEY TO CONTINUE");
     wrefresh(res_win);
     wgetch(res_win);  /* waits for any key pressed */

     delwin(lon_win);  /* Destroy longitude. latitude and results window */
     delwin(lat_win);
     delwin(res_win);
   }
  /* END OF CONVERT LONG/LAT TO LOCATOR SECTION*/






  /* PRINT WARRANTY SECTION*/
  if(choice=='3')
  {
  clear();
  attron(COLOR_PAIR(2));

  mvprintw(0,k+6," ***  IK2PIH LOCCALC V2.1 - a clone of SM0CCM LOCATOR CALCULATOR *** ");
  mvprintw(1,k+6,"                               WARRANTY                              ");
  attroff(COLOR_PAIR(2));
  mvprintw(4,k+5,"Loccalc, version 2.1.");
  mvprintw(5,k+5,"Copyright 2002, 2003 Marco Bersani, bermar2@inwind.it");
  mvprintw(7,k+5,"This program is free software; you can redistribute it and/or modify");
  mvprintw(8,k+5,"it under the terms of the GNU General Public License as published by");
  mvprintw(9,k+5,"the Free Software Foundation; either version 2 of the License, or");
  mvprintw(10,k+5,"(at your option) any later version.");
  mvprintw(12,k+5,"This program is distributed in the hope that it will be useful,");
  mvprintw(13,k+5,"but WITHOUT ANY WARRANTY; without even the implied warranty of");
  mvprintw(14,k+5,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
  mvprintw(15,k+5,"GNU General Public License for more details.");
  mvprintw(17,k+5,"You should have received a copy of the GNU General Public License");
  mvprintw(18,k+5,"along with this program (see the file COPYING which accompany");
  mvprintw(19,k+5,"this source file);  if not, write to the Free Software Foundation, Inc.,");
  mvprintw(20,k+5,"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA");
  mvprintw(22,k+25,"PRESS ANY KEY TO CONTINUE");
  refresh();
  getch();  /* waits for any key pressed */
  }
  /* END OF PRINT WARRANTY SECTION*/





  /* PRINT ABOUT SECTION*/
  if(choice=='4')
  {
  clear();
  attron(COLOR_PAIR(2));
  mvprintw(0,k+6," ***  IK2PIH LOCCALC V2.1 - a clone of SM0CCM LOCATOR CALCULATOR *** ");
  mvprintw(1,k+6,"                               ABOUT                                 ");
  attroff(COLOR_PAIR(2));
  mvprintw(4,k+2,"Loccalc is a program that calculates longitude and latitude from a 4 or 6");
  mvprintw(5,k+2,"characters world wide locator, calculates distance (kilometers and miles)");
  mvprintw(6,k+2,"between two locators and calculates a locator from longitude and latitude.");
  mvprintw(7,k+2,"You will be prompted for all inputs and all inputs are validity checked.");
  mvprintw(9,k+2,"Loccalc 2.1, written by Marco Bersani, IK2PIH, Stradella, Italy,");
  mvprintw(10,k+2,"on December 2003, is a clone of \"Locator Calculator V2.0\", fully re-written");
  mvprintw(11,k+2,"in C for the Linux OS; uses ncurses and requires a 80x24 color terminal.");
  mvprintw(13,k+2,"\"Locator Calculator V2.0\" was a free software written in GW-Basic");
  mvprintw(14,k+2,"by Lars Berg, SM0CCM,  Jarfalla, Sweden on 15 September 1987, and");
  mvprintw(15,k+2,"released by the author as a public domain software with no restrictions.");
  mvprintw(17,k+2,"Results are accurate for HAM purposes in a range of 2000 kilometers.");
  mvprintw(19,k+25,"PRESS ANY KEY TO CONTINUE");
  refresh();
  getch();  /* waits for any key pressed */
  }
/* END OF PRINT ABOUT SECTION*/


 }
while(choice!='5');


/* exit from ncurses */
endwin();


/* END OF MAIN ROUTINE */
return(0);
}













int check_valid_locator(char *locator)
{
/* -------------- Subroutine -----------------------
      Check valid locator
      Input : char *locator = 4 or 6 characters word wide locator.
      returned value ==  -1 No error. (Valid locator).
      returned value ==  0 Error.   (Invalid locator).
      Note: also string "END" is considered a valid locator, but returned value is -2.
   ------------------------------------------------- */
int lenght_of_locator,x,i;
char t;

lenght_of_locator = strlen(locator);


if (lenght_of_locator == 4)
            {strcat(locator,"LM");
             lenght_of_locator=6;}                 /* Adjust to middle of square */


x=-1;                                                /* Assumes locator is right */


if (strcmp(locator,"END")==0) x=-2;                  /* Return -2 if locator=END */

else if (lenght_of_locator != 6) x=0;                /* force to 0 if locator is wrong */

else for(i=0; i<=5; i++)
       {t=locator[i];
        if(((t > 'R') || (t < 'A')) && ((i==0) || (i==1))) x = 0;  /* first 2 characters: from A to R */
        if(((t > '9') || (t < '0')) && ((i==2) || (i==3))) x = 0;  /* check the numbers  */
        if(((t > 'X') || (t < 'A')) && ((i==4) || (i==5))) x = 0;  /* last 2 characters: from A to X */
       }

return(x);
}













void calculate_longitude_and_latitude_from_locator(char *locator, double *lon, double *lat)
{
/* -------------- Subroutine -----------------------
        calculate longitude and latitude
        Input : locator   = 6 characters world wide locator.
        Output: lon = longitude in decimal degrees;
                lat  = latitude in decimal degrees;
------------------------------------------------- */


*lon = -180+((double)locator[0]-65)*20 + ((double)locator[2]-48)*2 +((double)locator[4]-64.5)/12;
*lon = -(*lon);
*lat = -90+((double)locator[1]-65)*10 + ((double)locator[3]-48) +((double)locator[5]-64.5)/24;
return;
}












void calculate_beam_heading_and_distance(double lon1, double lat1, double lon2, double lat2, double *cb, double *di, double *dmi)
{
/* ------Subroutine--------------------
        Calculate beam heading and distance
   Input : lon1  = Longitude 1 in degrees.
         : lat1  = Latitude 1 in degrees.
         : lon2  = Longitude 2 in degrees.
         : lat2  = Latitude 2 in degrees.
   Output: cb   = Beam heading in degrees.
           di   = Distance in kilometers.
           dmi  = Distance in miles.
 ------------------------------------ */

double lo1,la1,lo2,la2,x,radius;

lo1=lon1*PI/180;   /* Convert degrees to radians */
la1=lat1*PI/180;
lo2=lon2*PI/180;
la2=lat2*PI/180;

/* Get local earth radius */
radius=local_earth_radius(lat1);

/* Calculates distance in km and in miles */
(*di)=acos(cos(la1)*cos(lo1)*cos(la2)*cos(lo2)+cos(la1)*sin(lo1)*cos(la2)*sin(lo2)+sin(la1)*sin(la2))*radius;
(*dmi)=(*di)/1.609;


/* Calculates beam heading */
x=atan2(sin(lo1-lo2)*cos(la2),cos(la1)*sin(la2)-sin(la1)*cos(la2)*cos(lo1-lo2))/PI*180;


if(x<0)
    (*cb)=x+360;
else
    (*cb)=x;


return;
}














void convert_character_to_upper_case(char *string)
{
int lenght;
int i;

/* ----------------SUBROUTINE-----------------------
       CONVERT CHARACTER TO UPPER CASE
       Input : string = Variable length. Upper and/or lower case.
       Output: string = Variable length. Upper case only.
   ------------------------------------------------- */

lenght=strlen(string);

if(lenght!=0)
 {
  for(i=0;i<lenght;i++)
  {
   if(string[i]>='a' && string[i]<='z')
         string[i]=string[i]-32;
  }
 }
return;
}










void calculate_locator_from_longitude_and_latitude(char *locator, double lon, double lat)
{
double lo,la;
int alo,bla,clo,dla,elo,fla;

/* -------------- Subroutine -----------------------
   Calculate locator from longitude and latitude
   Input : lon = Longitude in decimal degrees (+ = West;  - = East).
           lat = Latitude in decimal degrees (+ = North; - = South).
   Output: locator = 6 characters world wide locator.
   ------------------------------------------------- */

lo=(-lon+180)/20;
la=(lat+90)/10;

alo=(int)floor(lo);
bla=(int)floor(la);
lo=(lo-(double)alo)*10;
la=(la-(double)bla)*10;

clo = (int)floor(lo);
dla = (int)floor(la);

elo = (int)floor((lo-(double)clo)*24);
fla = (int)floor((la-(double)dla)*24);

locator[0]=(char)(alo+'A');
locator[1]=(char)(bla+'A');
locator[2]=(char)(clo+'0');
locator[3]=(char)(dla+'0');
locator[4]=(char)(elo+'A');
locator[5]=(char)(fla+'A');
locator[6]='\0';

return;
}









int get_decimal_degrees_from_input_string(char *inputstring, double *result, char t)
{
/* -------------- Subroutine -----------------------
      Get decimal degrees from input string
      Return 0 if error occurs; Return -1 if OK;
   ------------------------------------------------- */
char degs[3];
char fract_of_degree[2];
char c,d;
double dg;
double dec_dg;
double mins;
int i;



c=inputstring[0];

if((t==IS_LONGITUDE && c!='W' && c!='E') || (t==IS_LATITUDE && c!='N' && c!='S'))
                                                                        return(0);
/* Extract degrees from string and convert to double*/
for(i=1;i<=3;i++)
    {if(inputstring[i]>='0' && inputstring[i]<='9') degs[i-1]=inputstring[i];
     else return(0);}
degs[3]='\0';

sscanf(degs,"%lf",&dg);




/* Extract fraction of degrees from string and converts minutes to decimal degrees*/
for(i=5;i<=6;i++)
    {if(inputstring[i]>='0' && inputstring[i]<='9') fract_of_degree[i-5]=inputstring[i];
     else return(0);}
fract_of_degree[2]='\0';


d=inputstring[4];

if(d=='.')
      sscanf(fract_of_degree,"%lf",&dec_dg);

else if((d=='\'') && (fract_of_degree[0]<='5'))
     {sscanf(fract_of_degree,"%lf",&mins);
      dec_dg=mins/60*100;}

else return(0);


/* Build number in decimal degrees and checks result */
dg=dg+(dec_dg/100);
if((t==IS_LONGITUDE && dg>180) || (t==IS_LATITUDE && dg>90))
                                                return(0);


/* Minus if longitude east or latitude south */
if(c=='E' || c=='S') dg=-dg;

/* Export result */
*result=dg;

return(-1);
}



double local_earth_radius(double lat)
{
/* -------------- Subroutine -----------------------
   Calculate local earth radius from latitude
   Input : lat = Latitude in decimal degrees (+ = North; - = South)
   Output: earth radius (km)
   ------------------------------------------------- */

double esq,a,b,r,la,sla;

/* Hayford axes (1909) */
a=6378.388;   /* earth mayor axis (km) (equatorial axis) */
b=6356.912;   /* earth minor axis (km) (polar axis) */

la=lat*PI/180;  /* convert latitude in radians */

sla=sin(la);           /* calculates sinus of latitude */
esq=(a*a-b*b)/(a*a);   /* calculates eccentricity^2 */

r=a*sqrt(1-esq)/(1-esq*sla*sla);  /* calculates local radius (km)*/
return(r);
}
